import * as React from 'react';
import { FiasLocale } from './locale';
import { Fields, ExtraFields, FiasValue, FormValidation, APIProvider, FieldsSettings } from './types';
import { Address } from './models/Address';
export interface FiasProps {
    /**
     * Значение адреса. См. формат в примерах
     */
    value?: Partial<FiasValue>;
    error?: boolean;
    warning?: boolean;
    /**
     * Сообщение пользователю в режиме `error` или `warning`
     */
    feedback?: string;
    /**
     * Выводить ли текстовую интерпретацию адреса над ссылкой
     */
    showAddressText?: boolean;
    /**
     * Текст ссылки
     */
    label?: string;
    /**
     * Иконка рядом со ссылкой
     */
    icon?: React.ReactElement<any>;
    readonly?: boolean;
    /**
     * API URL. Существует тестовый
     * `https://api.testkontur.ru/fias/v1/` и боевой `https://api.kontur.ru/fias/v1/`
     */
    baseUrl?: string;
    /**
     * API instance. Если нет возможности использовать стандартный API.
     */
    api?: APIProvider;
    /**
     * Позволяет получить полный FiasValue после обработки входного `value`
     */
    onInit?: (value: FiasValue) => void;
    onChange?: (value: FiasValue) => void;
    onClose?: () => void;
    /**
     * Добавляет поле поиска адреса в произвольной форме
     */
    search?: boolean;
    /**
     * Количество отображаемых элементов в выпадающих списках
     */
    limit?: number;
    /**
     * Словарь текстовых констант. См. полный список ниже
     * @deprecated используйте LocaleProvider
     */
    locale?: FiasLocale;
    /**
     * Уровень критичности ошибок валидации полей адреса
     */
    formValidation?: FormValidation;
    /**
     * Разрешать ли сохранять неверефицированный (произвольный, не из базы) адрес
     */
    allowNotVerified?: boolean;
    /**
     * Версия базы данных ФИАС. Формат: "2018-10-22"
     */
    version?: string;
    /**
     * Настройка полей. Достаточно переопределить только нужные.
     * Внимание, не рекомендуется скрывать поля в произвольном порядке. Это может привести
     * к невозможности заполнения некоторых адресов.
     * Значение по умолчанию:
     *
     * ```ts
     *  {
     *     region:
     *     ...
     *     room: {
     *       visible: true
     *     },
     *     postalcode: {
     *       visible: false
     *     }
     *  }```
     *
     */
    fieldsSettings: FieldsSettings;
    countrySelector: boolean;
}
export interface FiasState {
    opened: boolean;
    address: Address;
    locale: FiasLocale;
    fieldsSettings: FieldsSettings;
}
export declare class Fias extends React.Component<FiasProps, FiasState> {
    static defaultProps: {
        showAddressText: boolean;
        error: boolean;
        warning: boolean;
        readonly: boolean;
        search: boolean;
        icon: JSX.Element;
        allowNotVerified: boolean;
        fieldsSettings: {};
        countrySelector: boolean;
    };
    state: FiasState;
    private theme;
    private api;
    private form;
    private readonly locale;
    constructor(props: FiasProps);
    readonly fieldsSettings: FieldsSettings;
    componentDidMount: () => void;
    componentDidUpdate: (prevProps: FiasProps, prevState: FiasState) => void;
    isFieldVisible(field: Fields | ExtraFields): boolean;
    render(): JSX.Element;
    private renderMain;
    private getLocaleMix;
    private renderModal;
    private init;
    private updateAddress;
    private updateLocale;
    private updateFieldsSettings;
    private handleOpen;
    private handleClose;
    private handleSave;
    private handleChange;
    private refForm;
}
export default Fias;
