"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = tslib_1.__importStar(require("react"));
var PropTypes = tslib_1.__importStar(require("prop-types"));
/**
 * Контейнер, расстояние между элементами в котором равно `gap`.
 */
var Gapped = /** @class */ (function (_super) {
    tslib_1.__extends(Gapped, _super);
    function Gapped() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Gapped.prototype.render = function () {
        if (this.props.vertical) {
            return this._renderVertical();
        }
        return this._renderHorizontal();
    };
    Gapped.prototype._renderVertical = function () {
        var subsequentItemStyle = {
            paddingTop: this.props.gap,
        };
        var isFirst = true;
        var children = React.Children.map(this.props.children, function (child) {
            if (!child) {
                return child;
            }
            var style = isFirst ? undefined : subsequentItemStyle;
            isFirst = false;
            return React.createElement("div", { style: style }, child);
        });
        return React.createElement("div", null, children);
    };
    Gapped.prototype._renderHorizontal = function () {
        var itemStyle = {
            display: 'inline-block',
            marginLeft: this.props.gap,
            marginTop: this.props.gap,
            verticalAlign: this.props.verticalAlign,
        };
        var children = React.Children.map(this.props.children, function (child, index) {
            if (!child) {
                return child;
            }
            return React.createElement("span", { style: itemStyle }, child);
        });
        var rootStyle = {
            paddingTop: 1,
        };
        var contStyle = {
            marginTop: -this.props.gap - 1,
            marginLeft: -this.props.gap,
        };
        return (React.createElement("div", { style: rootStyle },
            React.createElement("div", { style: contStyle }, children)));
    };
    Gapped.propTypes = {
        /**
         * Расстояние между элементами.
         */
        gap: PropTypes.number,
        /**
         * Располагать элементы вертикально.
         */
        vertical: PropTypes.bool,
        /**
         * Вертикальное выравнивание элементов.
         */
        verticalAlign: PropTypes.oneOf(['top', 'middle', 'baseline', 'bottom']),
    };
    Gapped.defaultProps = {
        gap: 10,
        verticalAlign: 'middle',
    };
    return Gapped;
}(React.Component));
exports.default = Gapped;
//# sourceMappingURL=Gapped.js.map