import { ButtonUse, ButtonSize } from '../Button/Button';
import * as React from 'react';
import * as PropTypes from 'prop-types';
import Item from './Item';
import { Nullable } from '../../typings/utility-types';
export interface ButtonParams {
    disabled?: boolean;
    label: React.ReactNode;
    onClick: () => void;
    onKeyDown: (event: React.KeyboardEvent<HTMLElement>) => void;
    opened: boolean;
    isPlaceholder: boolean;
}
export interface SelectProps<TValue, TItem> {
    /** @ignore */
    _icon?: React.ReactElement<any>;
    /** @ignore */
    _renderButton?: (params: ButtonParams) => React.ReactNode;
    defaultValue?: TValue;
    /** @ignore */
    diadocLinkIcon?: React.ReactElement<any>;
    disablePortal?: boolean;
    disabled?: boolean;
    error?: boolean;
    filterItem?: (value: TValue, item: TItem, pattern: string) => boolean;
    items?: Array<[TValue, TItem, React.ReactNode?] | TItem | React.ReactElement<any> | (() => React.ReactElement<any>)>;
    maxMenuHeight?: number;
    maxWidth?: React.CSSProperties['maxWidth'];
    menuAlign?: 'left' | 'right';
    menuWidth?: React.CSSProperties['width'];
    onChange?: (e: {
        target: {
            value: TValue;
        };
    }, value: TValue) => void;
    onClose?: () => void;
    onMouseEnter?: (e: React.MouseEvent<HTMLElement>) => void;
    onMouseLeave?: (e: React.MouseEvent<HTMLElement>) => void;
    onMouseOver?: (e: React.MouseEvent<HTMLElement>) => void;
    onKeyDown?: (e: React.KeyboardEvent<HTMLElement>) => void;
    onOpen?: () => void;
    placeholder?: React.ReactNode;
    renderItem?: (value: TValue, item?: TItem) => React.ReactNode;
    renderValue?: (value: TValue, item?: TItem) => React.ReactNode;
    areValuesEqual?: (value1: TValue, value2: TValue) => boolean;
    search?: boolean;
    value?: TValue;
    width?: number | string;
    warning?: boolean;
    use?: ButtonUse;
    size?: ButtonSize;
    onFocus?: React.FocusEventHandler<HTMLElement>;
    onBlur?: React.FocusEventHandler<HTMLElement>;
}
export interface SelectState<TValue> {
    opened: boolean;
    searchPattern?: string;
    value: Nullable<TValue>;
}
declare class Select<TValue = {}, TItem = {}> extends React.Component<SelectProps<TValue, TItem>, SelectState<TValue>> {
    static propTypes: {
        /**
         * Функция для сравнения `value` с элементом из `items`
         */
        areValuesEqual: PropTypes.Requireable<(...args: any[]) => any>;
        defaultValue: PropTypes.Requireable<any>;
        /**
         * Отключает использование портала
         */
        disablePortal: PropTypes.Requireable<boolean>;
        disabled: PropTypes.Requireable<boolean>;
        /**
         * Визуально показать наличие ошибки.
         */
        error: PropTypes.Requireable<boolean>;
        filterItem: PropTypes.Requireable<(...args: any[]) => any>;
        /**
         * Набор значений. Поддерживаются любые перечисляемые типы, в том числе
         * `Array`, `Map`, `Immutable.Map`.
         *
         * Элементы воспринимаются следующим образом: если элемент — это массив, то
         * первый элемент является значением , второй — отображается в списке,
         * а третий – комментарий;
         * если элемент не является массивом, то он используется и для отображения,
         * и для значения.
         *
         * Для вставки разделителя можно использовать `Select.SEP`.
         *
         * Вставить невыделяемый элемент со своей разметкой можно так:
         * ```
         * <Select ...
         *   items={[Select.static(() => <div>My Element</div>)]}
         * />
         * ```
         *
         * Чтобы добавить стандартный отступ для статического элемента:
         * ```
         * <Select.Item>My Element</Select.Item>
         * ```?
         */
        items: PropTypes.Requireable<object>;
        maxMenuHeight: PropTypes.Requireable<number>;
        maxWidth: PropTypes.Requireable<string | number>;
        placeholder: PropTypes.Requireable<PropTypes.ReactNodeLike>;
        /**
         * Функция для отрисовки элемента в выпадающем списке. Аргументы — *value*,
         * *item*.
         */
        renderItem: PropTypes.Requireable<(...args: any[]) => any>;
        /**
         * Функция для отрисовки выбранного элемента. Аргументы — *value*, *item*.
         */
        renderValue: PropTypes.Requireable<(...args: any[]) => any>;
        /**
         * Показывать строку поиска в списке.
         */
        search: PropTypes.Requireable<boolean>;
        value: PropTypes.Requireable<any>;
        width: PropTypes.Requireable<string | number>;
        onChange: PropTypes.Requireable<(...args: any[]) => any>;
        onMouseEnter: PropTypes.Requireable<(...args: any[]) => any>;
        onMouseLeave: PropTypes.Requireable<(...args: any[]) => any>;
        onMouseOver: PropTypes.Requireable<(...args: any[]) => any>;
        onKeyDown: PropTypes.Requireable<(...args: any[]) => any>;
    };
    static defaultProps: {
        renderValue: typeof renderValue;
        renderItem: typeof renderItem;
        areValuesEqual: typeof areValuesEqual;
        filterItem: typeof filterItem;
        use: string;
    };
    static Item: typeof Item;
    static SEP: () => JSX.Element;
    static static: (element: string | number | boolean | {} | React.ReactElement<any> | React.ReactNodeArray | React.ReactPortal | (() => React.ReactNode) | null | undefined) => {} | null | undefined;
    state: SelectState<TValue>;
    private theme;
    private readonly locale;
    private menu;
    private buttonElement;
    private getProps;
    componentDidUpdate(_prevProps: SelectProps<TValue, TItem>, prevState: SelectState<TValue>): void;
    render(): JSX.Element;
    /**
     * @public
     */
    open: () => void;
    /**
     * @public
     */
    close: () => void;
    /**
     * @public
     */
    focus: () => void;
    private renderMain;
    private renderLabel;
    private renderDefaultButton;
    private renderLinkButton;
    private renderMenu;
    private dropdownContainerGetParent;
    private focusInput;
    private refMenu;
    private toggle;
    private handleKey;
    private handleSearch;
    private select;
    private getValue;
    private mapItems;
    private getItemByValue;
    private buttonRef;
    private getButton;
}
declare function renderValue(value: any, item: any): any;
declare function renderItem(value: any, item: any): any;
declare function areValuesEqual(value1: any, value2: any): boolean;
declare function filterItem(value: any, item: any, pattern: string): boolean;
export default Select;
