import * as React from 'react';
import { FocusEventHandler, MouseEventHandler, ReactNode } from 'react';
import { TokenProps } from '../Token';
import { MenuItemState } from '../MenuItem';
import { TokenActions } from '../Token/Token';
export declare enum TokenInputType {
    WithReference = 0,
    WithoutReference = 1,
    Combined = 2
}
export interface TokenInputProps<T> {
    selectedItems: T[];
    onChange: (items: T[]) => void;
    onMouseEnter: MouseEventHandler<HTMLDivElement>;
    onMouseLeave: MouseEventHandler<HTMLDivElement>;
    onFocus: FocusEventHandler<HTMLInputElement>;
    onBlur: FocusEventHandler<HTMLInputElement>;
    autoFocus?: boolean;
    type?: TokenInputType;
    getItems?: (query: string) => Promise<T[]>;
    hideMenuIfEmptyInputValue?: boolean;
    renderItem: (item: T, state: MenuItemState) => React.ReactNode | null;
    renderValue: (item: T) => React.ReactNode;
    /**
     * Функция должна возвращать строковое представление токена
     * @default item => item
     */
    valueToString: (item: T) => string;
    renderNotFound?: () => React.ReactNode;
    valueToItem: (item: string) => T;
    toKey: (item: T) => string | number | undefined;
    placeholder?: string;
    delimiters?: string[];
    error?: boolean;
    warning?: boolean;
    disabled?: boolean;
    width?: string | number;
    renderToken?: (item: T, props: Partial<TokenProps & TokenActions>) => ReactNode;
    /**
     * @deprecated Use `renderToken` prop instead
     */
    renderTokenComponent?: (token: (props?: Partial<TokenProps>) => React.ReactElement<TokenProps>, value?: T) => React.ReactElement<TokenProps>;
}
export interface TokenInputState<T> {
    autocompleteItems?: T[];
    activeTokens: T[];
    inFocus?: boolean;
    inputValue: string;
    inputValueWidth: number;
    preventBlur?: boolean;
    loading?: boolean;
}
export default class TokenInput<T = string> extends React.PureComponent<TokenInputProps<T>, TokenInputState<T>> {
    static defaultProps: Partial<TokenInputProps<any>>;
    state: TokenInputState<T>;
    private theme;
    private input;
    private tokensInputMenu;
    private textHelper;
    private wrapper;
    componentDidMount(): void;
    componentDidUpdate(prevProps: TokenInputProps<T>, prevState: TokenInputState<T>): void;
    componentWillUnmount(): void;
    render(): JSX.Element;
    private renderMain;
    private hasValueInItems;
    private readonly showAddItemHint;
    private readonly type;
    private readonly delimiters;
    private readonly menuRef;
    private readonly isCursorVisible;
    private isCursorVisibleForState;
    private inputRef;
    private tokensInputMenuRef;
    private textHelperRef;
    private wrapperRef;
    private dispatch;
    private updateInputTextWidth;
    private handleInputFocus;
    private handleInputBlur;
    private isBlurToMenu;
    private handleWrapperMouseDown;
    private handleWrapperMouseUp;
    private handleCopy;
    private handleInputPaste;
    private tryGetItems;
    private handleKeyDown;
    private handleInputKeyDown;
    private moveFocusToLastToken;
    private focusInput;
    private handleWrapperKeyDown;
    private handleWrapperArrows;
    private handleWrapperArrowsWithoutShift;
    private handleWrapperArrowsWithShift;
    private handleChange;
    private handleAddItem;
    private handleRemoveToken;
    private handleTokenClick;
    private handleChangeInputValue;
    private highlightMenuItem;
    private renderTokenFields;
    private renderTokenSelector;
    /**
     * @deprecated
     */
    private renderTokenComponent;
    private renderToken;
}
