/**
 * Поиск всех фокусируемых элементов в переданном родителе
 * или на всей странице
 * @param {Element|Document|null} [parent=document] - Родительский элемент,
 * внутри которого осуществляется поиск.
 * @return {HTMLElement[]} - Массив найденных элементов
 */
export declare const getFocusableElements: (parent?: Element | Document | null) => HTMLElement[];
/**
 * Поиск первого фокусируемого элемента в переданном родителе
 * или на всей странице
 * @param {Element|Document|null} [parent=document] - Родительский элемент,
 * внутри которого осуществляется поиск.
 * @return {HTMLElement|null} - Найденный элемент или null
 */
export declare const getFirstFocusableElement: (parent?: Element | Document | null) => HTMLElement | null;
/**
 * Поиск следующего фокусируемого элемента в переданном родителе
 * или на всей странице
 * @param {HTMLElement} current - Текущий фокусируемый элемент
 * @param {Element|Document|null} [parent=document] - Родительский элемент,
 * внутри которого осуществляется поиск.
 * @param {boolean} [recursive=true] - Разрешает рекурсивный подъем вверх
 * по родителям
 * @return {HTMLElement|null} - Найденный элемент или null
 *
 * В случае поиска по всей странице, можно оставить parent === document,
 * тогда результат будет максимально точным, но возможны задержки
 * при большом кол-ве фокусируемых элементов на странице.
 * Если же передать parent === current.parentElement и recursive === true,
 * то поиск осуществится максимально быстро. Но может оказаться не учтенным
 * порядок элементов с [tabindex] > 0.
 */
export declare const getNextFocusableElement: (current: HTMLElement, parent?: Element | Document | null, recursive?: boolean) => HTMLElement | null;
/**
 * Поиск ближайшего фокусируемого элемента среди переданного и его родителей
 * @param {HTMLElement} current - Текущий элемент
 * @return {HTMLElement|null} - Найденный элемент или null
 *
 */
export declare const getClosestFocusableElement: (current: HTMLElement | null) => HTMLElement | null;
export default getFocusableElements;
