"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var variables_module_less_1 = tslib_1.__importDefault(require("../../../components/variables.module.less"));
var ThemeHelpers_1 = require("../ThemeHelpers");
var ColorFunctions_1 = tslib_1.__importDefault(require("../../styles/ColorFunctions"));
var DEFAULT_THEME = ThemeHelpers_1.defineInternalTheme(variables_module_less_1.default, {
    bgActive: {
        get: function () {
            return this.blueLight;
        },
    },
    borderColorFocus: {
        get: function () {
            return this.blueLight;
        },
    },
    borderColorError: {
        get: function () {
            return this.errorMain;
        },
    },
    borderColorWarning: {
        get: function () {
            return this.warningMain;
        },
    },
    btnFontSizeSmall: {
        get: function () {
            return this.fontSizeSmall;
        },
    },
    btnFontSizeMedium: {
        get: function () {
            return this.fontSizeMedium;
        },
    },
    btnFontSizeLarge: {
        get: function () {
            return this.fontSizeLarge;
        },
    },
    btnPaddingYSmall: {
        get: function () {
            return this.controlPaddingYSmall;
        },
    },
    btnPaddingYMedium: {
        get: function () {
            return this.controlPaddingYMedium;
        },
    },
    btnPaddingYLarge: {
        get: function () {
            return this.controlPaddingYLarge;
        },
    },
    btnDisabledBg: {
        get: function () {
            return this.bgDisabled;
        },
    },
    controlHeightSmall: {
        get: function () {
            var borderWidth = parseInt(this.controlBorderWidth, 10) || 0;
            var paddingYSmall = parseInt(this.controlPaddingYSmall, 10) || 0;
            var lineHeightSmall = parseInt(this.controlLineHeightSmall, 10) || 0;
            return 2 * borderWidth + 2 * paddingYSmall + lineHeightSmall + "px";
        },
    },
    controlHeightMedium: {
        get: function () {
            var borderWidth = parseInt(this.controlBorderWidth, 10) || 0;
            var paddingYMedium = parseInt(this.controlPaddingYMedium, 10) || 0;
            var lineHeightMedium = parseInt(this.controlLineHeightMedium, 10) || 0;
            return 2 * borderWidth + 2 * paddingYMedium + lineHeightMedium + "px";
        },
    },
    controlHeightLarge: {
        get: function () {
            var borderWidth = parseInt(this.controlBorderWidth, 10) || 0;
            var paddingYLarge = parseInt(this.controlPaddingYLarge, 10) || 0;
            var lineHeightLarge = parseInt(this.controlLineHeightLarge, 10) || 0;
            return 2 * borderWidth + 2 * paddingYLarge + lineHeightLarge + "px";
        },
    },
    linkHoverColor: {
        get: function () {
            return this.linkColor;
        },
    },
    linkDisabledColor: {
        get: function () {
            return this.textColorDisabled;
        },
    },
    dropdownMenuHoverBg: {
        get: function () {
            return this.bgActive;
        },
    },
    btnDisabledShadowColor: {
        get: function () {
            return this.borderColorGrayLight;
        },
    },
    btnDisabledShadow: {
        get: function () {
            return "0 0 0 1px " + this.btnDisabledShadowColor;
        },
    },
    btnDisabledShadowArrow: {
        get: function () {
            return "1px -1px 0 0 " + this.btnDisabledShadowColor;
        },
    },
    tabColorFocus: {
        get: function () {
            return this.borderColorFocus;
        },
    },
    tabColorError: {
        get: function () {
            return this.btnDangerBg;
        },
    },
    tabColorWarning: {
        get: function () {
            return this.btnPayBg;
        },
    },
    tabColorSuccess: {
        get: function () {
            return this.btnSuccessBg;
        },
    },
    tabColorPrimary: {
        get: function () {
            return this.btnPrimaryBg;
        },
    },
    tabColorHover: {
        get: function () {
            return this.borderColorFocusLight;
        },
    },
    tabColorHoverError: {
        get: function () {
            return ColorFunctions_1.default.lighten(this.tabColorError, '25%');
        },
    },
    tabColorHoverWarning: {
        get: function () {
            return ColorFunctions_1.default.lighten(this.tabColorWarning, '25%');
        },
    },
    tabColorHoverSuccess: {
        get: function () {
            return ColorFunctions_1.default.lighten(this.tabColorSuccess, '25%');
        },
    },
    tabColorHoverPrimary: {
        get: function () {
            return ColorFunctions_1.default.lighten(this.tabColorPrimary, '25%');
        },
    },
    calendarCellHoverBgColor: {
        get: function () {
            return this.bgActive;
        },
    },
    dateSelectMenuBg: {
        get: function () {
            return this.bgDefault;
        },
    },
    dateSelectMenuItemBgActive: {
        get: function () {
            return this.bgActive;
        },
    },
    dateSelectMenuItemBgDisabled: {
        get: function () {
            return this.bgDefault;
        },
    },
    dateSelectMenuItemFontActive: {
        get: function () {
            return this.textColorInvert;
        },
    },
    dateSelectMenuItemFontSelected: {
        get: function () {
            return this.textColorDefault;
        },
    },
    dateSelectMenuItemFontDisabled: {
        get: function () {
            return this.textColorDisabled;
        },
    },
    pagingForwardLinkColor: {
        get: function () {
            return this.linkColor;
        },
    },
    pagingForwardLinkDisabledColor: {
        get: function () {
            return this.linkDisabledColor;
        },
    },
    toggleBgActive: {
        get: function () {
            return this.toggleBgChecked;
        },
    },
    toggleBgError: {
        get: function () {
            return this.errorMain;
        },
    },
    toggleBgWarning: {
        get: function () {
            return this.warningMain;
        },
    },
    toggleFocusShadowColor: {
        get: function () {
            return this.borderColorFocus;
        },
    },
    inputFontSizeSmall: {
        get: function () {
            return this.fontSizeSmall;
        },
    },
    inputFontSizeMedium: {
        get: function () {
            return this.fontSizeMedium;
        },
    },
    inputFontSizeLarge: {
        get: function () {
            return this.fontSizeLarge;
        },
    },
    inputFocusShadow: {
        get: function () {
            return "0 0 0 1px " + this.borderColorFocus;
        },
    },
    inputDisabledBg: {
        get: function () {
            return this.bgDisabled;
        },
    },
    inputDisabledBorderColor: {
        get: function () {
            return this.borderColorGrayLight;
        },
    },
    inputFocusOutline: {
        get: function () {
            return this.borderColorFocus;
        },
    },
    inputBorderWidth: {
        get: function () {
            return this.controlBorderWidth;
        },
    },
    inputBorderTopColor: {
        get: function () {
            return this.borderColorGrayDark;
        },
    },
    chbBorderColorFocus: {
        get: function () {
            return this.borderColorFocus;
        },
    },
    chbBg: {
        get: function () {
            return "linear-gradient(" + this.chbBgStart + ", " + this.chbBgEnd + ")";
        },
    },
    chbHoverBg: {
        get: function () {
            return "linear-gradient(-180deg, " + this.btnDefaultHoverBgStart + " 0, " + this.btnDefaultHoverBgEnd + " 100%)";
        },
    },
    chbActiveBg: {
        get: function () {
            return this.btnDefaultActiveBg;
        },
    },
    chbShadowActive: {
        get: function () {
            return this.btnDefaultActiveShadow;
        },
    },
    textareaLineHeight: {
        get: function () {
            var fontSize = this.fontSizeMedium;
            switch (fontSize) {
                case '14px':
                    return '20px';
                case '16px':
                    return '25px';
                default:
                    return parseInt(fontSize, 10) * 1.5 + "px";
            }
        },
    },
    textareaBorderTopColor: {
        get: function () {
            return this.borderColorGrayDark;
        },
    },
    textareaDisabledBg: {
        get: function () {
            return this.bgDisabled;
        },
    },
    textareaDisabledBorderColor: {
        get: function () {
            return this.borderColorGrayLight;
        },
    },
    spinnerBgColor: {
        get: function () {
            return this.grayXLight;
        },
    },
    spinnerDimmedColor: {
        get: function () {
            return this.gray;
        },
    },
    spinnerCaptionColor: {
        get: function () {
            return this.gray;
        },
    },
    popupBackground: {
        get: function () {
            return this.bgDefault;
        },
    },
});
exports.default = DEFAULT_THEME;
//# sourceMappingURL=DefaultTheme.js.map